// License: BSD-3-Clause  # Copyright 2015 Wm Leler

// Source: https://gist.github.com/wmleler/a734fb2bb3319a2cb386

// Get JSON data
//d3.json('data-flare.json', function(error, treeData) {

// Modifications by Jomin Yu, 2019:
//	Added touch support for panning, zooming and rotating via the root axis
//	Added regular tree, non-radial, and made the two switchable

const TYPE_NORMAL = 0;
const TYPE_RADIAL = 1;
var type = TYPE_NORMAL;
var expandNode = true;
if (storage) {
	var val = localStorage.getItem("people.tree.type");
	if (val != null) {
		type = val;
	}
	val = localStorage.getItem("people.tree.expand");
	if (val != null) {
		expandNode = val == 'true';
	}
}

function treeDual() {

	var treeData = data;
	// Calculate total nodes, max label length
	var totalNodes = 0;
	var maxLabelLength = 0;
	var DURATION = 700; // d3 animation duration
	var STAGGERN = 4; // delay for each node
	var STAGGERD = 200; // delay for each depth
	var NODE_DIAMETER = 7; // diameter of circular nodes
	var MIN_ZOOM = 0.5; // minimum zoom allowed
	var MAX_ZOOM = 10;  // maximum zoom allowed
	//var HAS_CHILDREN_COLOR = 'lightsteelblue';
	var SELECTED_COLOR = '#a00';  // color of selected node
	var ZOOM_INC = 0.04;  // zoom factor per animation frame
	var PAN_INC = 3;  //  pan per animation frame
	var ROT_INC = 0.3;  // rotation per animation frame

	var counter = 0;  // node ids
	var curNode;  // currently selected node
	var curPath;  // array of nodes in the path to the currently selected node

	// size of the diagram
	var screenWidth, screenHeight, width, height, minHeight;
	width = window.innerWidth;
	height = window.innerHeight;
	screenWidth = width;
	screenHeight = height;
	// my Samsung A7 sometimes has width/height as zero
	if (width == 0) {
		if (storage) {
			var val = localStorage.getItem("people.tree.width");
			if (val != null) {
				width = parseInt(val);
				screenWidth = width;
			}
			val = localStorage.getItem("people.tree.height");
			if (val != null) {
				height = parseInt(val);
				screenHeight = height;
			}
		}
		if (width == 0) {
			width = 360;
			height = 360;
			screenWidth = width;
			screenHeight = height;
		}
	} else if (storage) {
		localStorage.setItem("people.tree.width", width);
		localStorage.setItem("people.tree.height", height);
	}
	if (type == TYPE_RADIAL) {
		width -= 20;
		height -= 20;
	} else {
		minHeight = parseInt(height * 3 / 4);
	}
	
	// current pan, zoom, and rotation
	var curX = width / 2;
	var curY = height / 2;
	var curZ = 1.0; // current zoom
	var curR = 270; // current rotation

	// d3 diagonal projection for use by the node paths
	var diagonal;
	if (type == TYPE_RADIAL) {
		diagonal = d3.svg.diagonal.radial()
			.projection(function(d) {
				return [d.y, d.x / 180 * Math.PI];
			});
	} else {
		diagonal = d3.svg.diagonal()
			.projection(function(d) {
				return [d.y, d.x];
			});
	}
	
	var expandedAll = false;
	var butt = document.getElementById('type');
	butt.onclick = function () {
		if (type == TYPE_RADIAL) {
			type = TYPE_NORMAL;
		} else {
			type = TYPE_RADIAL;
		}
		if (storage) {
			localStorage.setItem("people.tree.type", type);
		}
		//location.reload();
		location = 'reload';
	};
	if (type == TYPE_RADIAL)
		butt.innerHTML = normalTreeSVG;
	else
		butt.innerHTML = radialTreeSVG;
	var tapmode = document.getElementById('tapmode');
	tapmode.onclick = function () {
		expandNode = !expandNode;
		if (storage) {
			localStorage.setItem("people.tree.expand", expandNode);
		}
		if (expandNode)
			tapmode.innerHTML = expandTreeSVG;
		else
			tapmode.innerHTML = infoTreeSVG;
	};
	if (expandNode)
		tapmode.innerHTML = expandTreeSVG;
	else
		tapmode.innerHTML = infoTreeSVG;
	butt = document.getElementById('expandall');
	butt.onclick = function (e) {
		$('#dialogExpand').dialog('open');
		e.preventDefault();
	};
	butt = document.getElementById('settings');
	butt.onclick = function (e) {
		$('#dialogSettings').dialog('open');
		e.preventDefault();
	};
	var animate = true;
	if (storage) {
		var val = localStorage.getItem("people.tree.animate");
		if (val != null) {
			animate = val == 'true';
		}
	}
	butt = document.getElementById('help');
	butt.onclick = function () {
		location = 'help';
	};
	var darkMode = false;
	if (storage) {
		var val = localStorage.getItem("people.tree.dark");
		if (val != null) {
			darkMode = val == 'true';
		}
	}
	changeDarkMode(darkMode);
	
	// A recursive helper function for performing some setup by walking through all nodes
	function visit(parent, visitFn, childrenFn) {
		if (!parent) return;

		visitFn(parent);

		var children = childrenFn(parent);
		if (children) {
			var count = children.length;
			for (var i = 0; i < count; i++) {
				visit(children[i], visitFn, childrenFn);
			}
		}
	}

	// Call visit function to establish maxLabelLength
	visit(treeData, function(d) {
		totalNodes++;
		maxLabelLength = Math.max(d.name.length, maxLabelLength);

	}, function(d) {
		return d.children && d.children.length > 0 ? d.children : null;
	});
	// d3 tree layout
	var tree;
	if (type == TYPE_RADIAL) {
		if (maxLabelLength > 10)
			maxLabelLength = 10;
		tree = d3.layout.tree()
			// .nodeSize([4.5, 120])
			.size([360, Math.min(width, height) / 2 - 120])
			.separation(function(a, b) {
				return a.depth === 0 ? 1 : (a.parent === b.parent ? 1 : 2) / a.depth;
		});
	} else {
		if (maxLabelLength > 12)
			maxLabelLength = 12;
		tree = d3.layout.tree()
			.size([height, width]);
	}

	// define the svgBase, attaching a class for styling and the zoomListener
	var svgBase = d3.select('#tree-container').append('svg')
		.attr('width', width)
		.attr('height', height)
		.on('mousedown', mousedown)
		.on('touchstart', mousedown);

	// Group which holds all nodes and manages pan, zoom, rotate
	var svgGroup = svgBase.append('g')
		.attr('transform', 'translate(' + curX + ',' + curY + ')');

	// Define the data root
	var root = treeData;
	if (type == TYPE_RADIAL)
		root.x0 = curY;
	else
		root.x0 = height / 2;
	root.y0 = 0;
	selectNode(root); // current selected node

	// Collapse all children of root's children before rendering
	// if (root.children) {
	//   root.children.forEach(function(child) {
	//       collapseTree(child);
	//   });
	// }
	
	// hide far ancestor's grandchildren
	function simplify(d) {
		if (d.hi >= 1 && d.children) {
			d.children.forEach(function(d) {
				if (d.hi == 1)
					simplify(d)
				else if (d.hi == 2 && d.children) {
					d.children.forEach(function(d) {
						collapseTree(d)
					});
				} else
					collapseTree(d)
			});
		}
	}
	simplify(data);
	
	// hide grandfather's ancestors
	var hide = true;
	var centered = false;
	function simplify2(d) {
		if (d.child) {
			var c1 = d.child;
			if (c1.child) {
				var c2 = c1.child;
				if (c2.hi == 2) {
					toggle(d);
					//alert(c3.name)
					selectNode(target);
					update(d, true);
					centerNode(target);
					centered = true;
				} else {
					if (hide)
						toggle(d);
					hide = !hide;
					simplify2(c1);
				}
			}
			
		}
	}
	// make sure parent property is available
	setTimeout(function() {
		simplify2(data);
		if (!centered) {
			selectNode(target);
			update(target, true);
			centerNode(target);
		}
	}, 32);

	update(root, true); // Layout the tree initially and center on the root node for radial

	var levels;
	
	// update the tree
	// source - source node of the update
	// transition - whether to do a transition
	function update(source, transition) {
		
		if (!animate)
			transition = false;
		if (type == TYPE_NORMAL) {
			// compute the new height
			var levelWidth = [1];
			var childCount = function(level, n) {
				if (n.children && n.children.length > 0) {
					if (levelWidth.length <= level + 1) levelWidth.push(0);
					levelWidth[level + 1] += n.children.length;
					n.children.forEach(function(d) {
						childCount(level + 1, d);
					});
				}
			};
			childCount(0, root);
			levels = levelWidth.length;
			var maxWidth = d3.max(levelWidth);
			if (maxWidth <= 20)
				height = maxWidth * 25;
			else
				height = maxWidth * (25 + (maxWidth - 20) / 1.7);
			if (height < minHeight)
				height = minHeight;
			tree = tree.size([height, width]);
		}

		var duration = transition ? DURATION : 0;

		// Compute the new tree layout.
		var nodes = tree.nodes(root);
		var links = tree.links(nodes);

		// Set widths between levels based on maxLabelLength.
		//var s = '';
		nodes.forEach(function(d) {
			d.ox = d.x; // original x
		});
		nodes.forEach(function(d) {
			d.y = (d.depth * (maxLabelLength * 10)); //maxLabelLength * 10px
			// alternatively to keep a fixed scale one can set a fixed depth per level
			// Normalize for fixed-depth by commenting out below line
			// d.y = (d.depth * 500); //500px per level.
			if (type == TYPE_RADIAL) {
				var p = d.parent;
				var n = d.depth > 0 ? p.children.length : 0;
				if (d.depth >= 4) {
					if (n > 1)
						d.x = p.x + (d.x - p.ox) * 4 * Math.pow(d.depth * 2, -0.75 + d.depth / 350);
					else
						d.x = p.x;
					//d.x = d.ox;
				}
				//if (d.name=="Asa")alert(p.x+' '+d.x);
			}
		});
		//if (s != '') alert(s);

		// Update the view
		svgGroup.transition().duration(duration)
			.attr('transform',
				(type == TYPE_RADIAL ? 'rotate(' + curR + ' ' + curX + ' ' + curY + ')' : '') +
				'translate(' + curX + ' ' + curY +
				')scale(' + curZ + ')');
		
		// new duration based on depth
		if (source.depth < 5)
			duration *= 1 / (source.depth * 2 + 1);
		else
			duration = 0;

		// Update the nodes…
		var node = svgGroup.selectAll('g.node')
			.data(nodes, function(d) {
				return d.id || (d.id = ++counter);
			});

		// Enter any new nodes at the parent's previous position
		var nodeEnter = node.enter().insert('g', ':first-child')
			.attr('class', 'node')
			.attr('transform',
				(type == TYPE_RADIAL ? 'rotate(' + (source.x0 - 90) + ')translate(' + source.y0 + ')' :
				'translate(' + source.y0 + ',' + source.x0 + ')'))
			.on('click', click);//.on('dblclick', dblclick).on('contextmenu', showContextMenu);
			// .on('mousedown', suppress);

		nodeEnter.append('circle')
			.attr('r', 1e-6)
			//.style('fill', function(d) {
			//	return d._children ? HAS_CHILDREN_COLOR : 'white';
			//});

		nodeEnter.append('text')
			.text(function(d) {
				return d.name;
			})
			.style('opacity', 0.9)
			.style('fill-opacity', 0)
			.attr('transform', function() {
				if (type == TYPE_RADIAL)
					return ((source.x0 + curR) % 360 <= 180 ?
						'translate(8)scale(' :
						'rotate(180)translate(-8)scale('
					) + reduceZ() + ')';
				else
					return 'translate(8)scale(' + reduceZ() + ')';
			});

		// update existing graph nodes

		// Change the circle fill depending on whether it has children and is collapsed
		node.select('circle')
			.attr('r', function(d) {
				return (NODE_DIAMETER + (d.hi == 2 ? 2 : 0)) * reduceZ();
			})
			.attr('class', function(d) {
				var cls = d.hi == 1 ? 'red' : (d.hi == 2 ? 'red2' : 'nc');
				if (d._children)
					cls += ' chil';
				if (d.selected)
					cls += ' sel';
				return cls;
			});

		node.select('text')
			.attr('x', function(d) {
				if (type == TYPE_RADIAL)
					return (d.x + curR) % 360 <= 180 ? 2 : -2;
				else
					return 2;
			})
			.attr('text-anchor', function(d) {
				if (type == TYPE_RADIAL)
					return (d.x + curR) % 360 <= 180 ? 'start' : 'end';
				else
					return 'start';
			})
			.attr('transform', function(d) {
				if (type == TYPE_RADIAL)
					return ((d.x + curR) % 360 <= 180 ?
						'translate(8)scale(' :
						'rotate(180)translate(-8)scale('
					  ) + reduceZ() +')';
				else
					return 'translate(8)scale(' + reduceZ() +')';
			}).attr('fill', function(d) {
				return d.selected ? SELECTED_COLOR : 'black';
			}).attr('dy', '.35em');

		var nodeUpdate = node.transition().duration(duration)
			.delay( transition ? function(d, i) {
				return i * STAGGERN +
					/*Math.abs(d.depth - curNode.depth) **/ STAGGERD; }  : 0)
			.attr('transform', function(d) {
				if (type == TYPE_RADIAL)
					return 'rotate(' + (d.x - 90) + ')translate(' + d.y + ')';
				else
					return 'translate(' + d.y + ',' + d.x + ')';
			});

		nodeUpdate.select('circle')
			.attr('r', function(d) {
				return (NODE_DIAMETER + (d.hi == 2 ? 2 : 0)) * reduceZ();
			//}).style('fill', function(d) {
			//   return d._children ? HAS_CHILDREN_COLOR : 'white';
			});

		nodeUpdate.select('text')
			.style('fill-opacity', 1);

		// Transition exiting nodes to the parent's new position and remove
		var nodeExit = node.exit().transition().duration(duration)
			.delay( transition ? function(d, i) {
				return i * STAGGERN; } : 0)
			.attr('transform', function() {
				if (type == TYPE_RADIAL)
					return 'rotate(' + (source.x - 90) + ')translate(' + source.y + ')';
				else
					return 'translate(' + source.y + ',' + source.x + ')';
		}).remove();

		nodeExit.select('circle').attr('r', 0);
		nodeExit.select('text').style('fill-opacity', 0);

		// Update the links…
		var link = svgGroup.selectAll('path.link')
			.data(links, function(d) {
				return d.target.id;
			});

		// Enter any new links at the parent's previous position
		link.enter().insert('path', 'g')
			.attr("class", function(d) {
				return d.target.hi > 0 ? "link redlink" : "link";
			})
			.attr('d', function() {
			var o = {
				x: source.x0,
				y: source.y0
			};
			return diagonal({
				source: o,
				target: o
			});
		});

		// Transition links to their new position
		link.transition().duration(duration)
			.delay( transition ? function(d, i) {
				return i * STAGGERN +
					/*Math.abs(d.source.depth - curNode.depth) **/ STAGGERD;
					// Math.max(0, d.source.depth - curNode.depth) * STAGGERD;
				} : 0)
			.attr('d', diagonal);

		// Transition exiting nodes to the parent's new position
		link.exit().transition().duration(duration)
			.attr('d', function() {
				var o = {
					x: source.x0,
					y: source.y0
				};
				return diagonal({
					source: o,
					target: o
				});
			}).remove();

		// Stash the old positions for transition
		nodes.forEach(function(d) {
			d.x0 = d.x;
			d.y0 = d.y;
		});
	} // end update

	// Helper functions for collapsing and expanding nodes
	var ancestryExpanded;

	// Toggle expand / collapse
	function toggle(d) {
		ancestryExpanded = false;
		if (d._children) {
			d.children = d._children;
			d._children = null;
			if (d.child) {
				d.parent = d._parent
				d._parent = null;
				root = d.root;
				d.root = null;
				ancestryExpanded = true;
			}
		} else if (d.children) {
			d._children = d.children;
			if (d.child) {
				d.children =[d.child]
				d._parent = d.parent
				d.root = root;
				d.parent = null;
				root = d;
			} else
				d.children = null;
		}
		return d;
	}

	// Expand ancestry
	function expandAncestry(d) {
		if (d._children) {
			d.children = d._children;
			d._children = null;
			if (d.child) {
				d.parent = d._parent
				d._parent = null;
				root = d.root;
				d.root = null;
				if (root._children)
					expandAncestry(root);
			}
		}
		return d;
	}

	function toggleTree(d) {
		if (d.children) {
			collapseTree(d);
		} else {
			expandTree(d);
		}
	}

	function expand(d) {
		if (d._children) {
			d.children = d._children;
			d._children = null;
		}
	}

	// expand all children, whether expanded or collapsed
	function expandTree(d) {
		if (d._children) {
			d.children = d._children;
			d._children = null;
		}
		if (d.children) {
			d.children.forEach(expandTree);
		}
	}

	// expand all children, whether expanded or collapsed up to target only
	function expandTreeToTarget(d) {
		if (d._children) {
			d.children = d._children;
			d._children = null;
		}
		if (d.children && d.hi != 2) {
			d.children.forEach(expandTreeToTarget);
		}
	}

	function collapse(d) {
		if (d.children) {
			d._children = d.children;
			d.children = null;
		}
	}

	// collapse all children
	function collapseTree(d) {
		if (d.children) {
			d._children = d.children;
			d.children = null;
		}
		if (d._children) {
			d._children.forEach(collapseTree);
		}
	}

	// expand one level of tree
	function expand1Level(d) {
		var q = [d]; // non-recursive
		var cn;
		var done = null;
		while (q.length > 0) {
			cn = q.shift();
			if (done !== null && done < cn.depth) { return; }
			if (cn._children) {
				done = cn.depth;
				cn.children = cn._children;
				cn._children = null;
				cn.children.forEach(collapse);
			}
			if (cn.children) { q = q.concat(cn.children); }
		}
		// no nodes to open
	}

	// highlight selected node
	function selectNode(node) {
		if (curNode) {
			delete curNode.selected;
		}
		curNode = node;
		curNode.selected = true;
		curPath = []; // filled in by fullpath
		//d3.select('#selection').html(fullpath(node));
	}

	// d3 event handlers

	function switchroot() {
		d3.event.preventDefault();
		var pathelms = document.querySelectorAll('#selection .nodepath');
		for (var i = 0; i < pathelms.length; i++) {
			pathelms[i].classList.remove('highlight');
		}
		var target = d3.event.target;
		var node = curPath[+target.dataset.sel];
		if (d3.event.shiftKey) {
			if (curNode !== node) {
			  selectNode(node);
			}
		} else {
			root = node;
			target.classList.add('highlight');
		}
		update(root, true);
	}

	function click(d) { // select node
		// commented out as Windows Chrome browser prevented re-opening
		//if (d3.event.defaultPrevented || d === curNode) { return; } // suppressed
		//d3.event.preventDefault();
		//selectNode(d);
		//update(d);
		var expand = expandNode;
		if (expand) {
			if (!d._children && !d.children || !d.parent && d.children && d.children.length == 1 && !d._children)
				expand = false;
		}
		if (expand) {
			toggle(d);
			centerNode(d);
			if (expandedAll || !ancestryExpanded) {
				setTimeout(function() {
					centerNode(d);
				}, 32);
			}
		} else {
			//alert(d.name);
			location = 'op' + d.id;	// person info
		}
	}
	
	function centerNode(d) {
		if (type == TYPE_RADIAL) {
			curX = width / 2 - d.y * curZ;
			curY = height / 2;
		} else {
			curX = screenWidth / 2 - d.y * curZ;
			curY = screenHeight / 2 - d.x * curZ;
		}
		curR = limitR(90 - d.x);
		update(d, true);
	}

	var startposX, startposY; // initial position on mouse button down for pan
	var sx1, sy1, sx2, sy2;
	var midx, midy, offX, offY, corX;
	var startZ, startR;
	var ignoreInit = false;

	function mousedown() {	// pan
		//d3.event.preventDefault();
		//if (d3.event.which !== 1 || d3.event.ctrlKey) { return; } // ingore other mouse buttons
		//console.log(d3.event);
		if (d3.event.clientX) {
			startposX = curX - d3.event.clientX;
			startposY = curY - d3.event.clientY;
			d3.select(document).on('mousemove', mousemove, true);
			d3.select(document).on('mouseup', mouseup, true);
		} else {
			startposX = curX - d3.event.touches[0].clientX;
			startposY = curY - d3.event.touches[0].clientY;
			if (d3.event.touches.length > 1) {
				sx1 = d3.event.touches[0].pageX;
				sy1 = d3.event.touches[0].pageY;
				sx2 = d3.event.touches[1].pageX;
				sy2 = d3.event.touches[1].pageY;
				startZ = curZ;
				ignoreInit = true;
				startR = Math.atan2(sy2 - sy1, sx2 - sx1);
				
				corX = curX - width / 2;
				//midx = ((d3.event.touches[0].clientX + d3.event.touches[1].clientX) / 2 - curX) / curZ;
				//midy = ((d3.event.touches[0].clientY + d3.event.touches[1].clientY) / 2 - curY) / curZ;
				midx = (d3.event.touches[0].clientX + d3.event.touches[1].clientX) / 2;
				midy = (d3.event.touches[0].clientY + d3.event.touches[1].clientY) / 2;
				offX = (midx - curX) / curZ;
				offY = (midy - curY) / curZ;
				//console.log("start: " + d3.event.touches[0].clientX + "," + d3.event.touches[1].clientX)
				console.log("mid: " + midx + "," + midy + " /// " + curX + "," + curY)
			}
			d3.select(document).on('touchmove', mousemove, true);
			d3.select(document).on('touchend', mouseup, true);
		}
		//console.log("startpos: " + startposX + "," + startposY);// + " ; " + startposX2 + "," + startposY2)
	}

	function mousemove() {
		//d3.event.preventDefault();
		var x = (d3.event.clientX || d3.event.touches[0].clientX);
		var y = (d3.event.clientY || d3.event.touches[0].clientY);
		if (d3.event.touches && d3.event.touches.length > 1) {
			var x1 = d3.event.touches[0].pageX;
			var y1 = d3.event.touches[0].pageY;
			var x2 = d3.event.touches[1].pageX;
			var y2 = d3.event.touches[1].pageY;
			var curX2 = sx2 + x2;
			var curY2 = sy2 + y2;
			// add sqrt later
			var l1 = Math.abs(Math.sqrt((sy2 - sy1) * (sy2 - sy1) + (sx2 - sx1) * (sx2 - sx1)));
			var l2 = Math.abs(Math.sqrt((y2 - y1) * (y2 - y1) + (x2 - x1) * (x2 - x1)));
			console.log(l1 + " " + l2 + " = " + (startZ * l2 / l1));
			var newZ = startZ * l2 / l1;
			if (Math.abs((newZ - curZ) / newZ) > 0.001) {
				curZ = newZ;
				//curX = (startposX + x)// * (1 - newZ);
				//curY = (startposY + y)// * (1 - newZ);
			}
			var xx = d3.event.touches[1].clientX;
			var yy = d3.event.touches[1].clientY;
			//curR = limitR(90 - x);
			if (type == TYPE_RADIAL) {
				var endR = Math.atan2(y2 - y1, x2 - x1);
				var newR = curR + (endR - startR) * 180 / Math.PI;
				if (Math.abs((newR - curR) / newR) > 0.001) {
					curR = newR;
					startR = endR;
				}
			} //else {
			//curX = width / 2 - midx * curZ;
			//curY = height / 2 - midy * curZ;
			curX = midx - offX * curZ;
			curY = midy - offY * curZ;
			//}
		} else {
			curX = startposX + x;
			curY = startposY + y;
		}
		//console.log("move: " + x + "," + y + " ; " + x2 + "," + y2)
		//console.log(curX + " " + curY);
		setview();
	}

	function mouseup() {
		if (d3.event.clientX) {
			d3.select(document).on('mousemove', null);
			d3.select(document).on('mouseup', null);
		} else {
			d3.select(document).on('touchmove', null);
			d3.select(document).on('touchend', null);
		}
	}

	// keep rotation between 0 and 360
	function limitR(r) {
		return (r + 360) % 360;
	}

	// limit size of text and nodes as scale increases
	function reduceZ() {
		return Math.pow(1.1, -curZ);
	}

	// set view with no animation
	function setview() {
		svgGroup.attr('transform',
			(type == TYPE_RADIAL ? 'rotate(' + curR + ' ' + curX + ' ' + curY + ')' : '') +
			'translate(' + curX + ' ' + curY + ')scale(' + curZ + ')');
		svgGroup.selectAll('text')
			.attr('text-anchor', function(d) {
				if (type == TYPE_RADIAL)
					return (d.x + curR) % 360 <= 180 ? 'start' : 'end';
				else
					return 'start';
			})
			.attr('transform', function(d) {
				if (type == TYPE_RADIAL)
					return ((d.x + curR) % 360 <= 180 ?
						'translate(8)scale(' :
						'rotate(180)translate(-8)scale('
						) + reduceZ() +')';
				else
					return 'translate(8)scale(' + reduceZ() +')';
			});
		svgGroup.selectAll('circle').attr('r', NODE_DIAMETER * reduceZ());
	}
	
	function expandParams(ancestry, ancestryDesc, descendants) {
		if (!ancestry && !ancestryDesc && !descendants)
			return;
		if (ancestryDesc && descendants) {
			expandAncestry(root);
			expandTree(root);
			update(root, true);
			centerNode(target);
			expandedAll = true;
			return;
		}
		if (!ancestryDesc) {
			if (ancestry) expandAncestry(root);
			if (descendants) {
				expandTree(root);
				if (!ancestry) root._children = root.children;
			}
			update(root, true);
			centerNode(target);
			expandedAll = true;
			return;
		}
		if (ancestryDesc) {
			expandAncestry(root);
			expandTreeToTarget(root);
			update(root, true);
			centerNode(target);
			expandedAll = true;
			return;
		}
	}
	
	function changeMode(pAnimate, pDark, pExpand) {
		animate = pAnimate;
		darkMode = pDark;
		expandNode = pExpand;
		if (expandNode)
			tapmode.innerHTML = expandTreeSVG;
		else
			tapmode.innerHTML = infoTreeSVG;
	}
	
	var obj = new Object();
	obj.expandParams = expandParams;
	obj.changeMode = changeMode;
	
	return obj;
}

var tree;
$( document ).ready(function() {
	tree = treeDual();
});

function getTarget(d) {
	if (d.id == id) {
		// hi is 2 for target child, ancestors are 1
		d.hi = 2;
		return d;
	}
	if (d.children) {
		for (var i = 0; i < d.children.length; i++) {
			var child = d.children[i];
			var ret = getTarget(child);
			if (ret != null) {
				// hi is 2 for target child, ancestors are 1
				// child is node to be retained in collapse
				d.hi = 1;
				d.child = child;
				return ret;
			}
		}
	}
	return null;
}
